package org.hypik.test.jboss.eap7.server;

import org.infinispan.Cache;
import org.infinispan.manager.EmbeddedCacheManager;
import org.jboss.msc.service.StartException;
import org.slf4j.Logger;

import javax.annotation.PostConstruct;
import javax.annotation.Resource;
import javax.ejb.Local;
import javax.ejb.Schedule;
import javax.ejb.Singleton;
import javax.ejb.Startup;
import javax.inject.Inject;
import javax.naming.NamingException;

@Singleton
@Startup
@Local(ITestService.class)
public class TestService implements ITestService {
  
  /**
   * serila uid.
   */
  private static final long serialVersionUID = 627547552200191670L;

  @Inject
  private Logger logger;

  @Resource(lookup = "java:jboss/infinispan/container/my-cache-container")
  private EmbeddedCacheManager cacheContainer;

  private Cache<Long, Long> myAppCache;

  @PostConstruct
  public void startNotClustered() throws NamingException {
    this.myAppCache = this.cacheContainer.getCache("myAppCache");
    
    for (String name : cacheContainer.getCacheNames()) {
      logger.info("Cache {} is {}",
          name, this.cacheContainer.getCache(name).getCacheConfiguration());
    }
  }

  @Override
  public void startClusteredSingleton() throws StartException {
    logger.info("Singleton Clustered Start of Test Service on node {}",
          System.getProperty("jboss.node.name"));
    try {
      Thread.sleep(10000);
    } catch (Exception e) {
      throw new StartException(e);
    }
    logger.info("Singleton Clustered Started of Test Service on node {}",
        System.getProperty("jboss.node.name"));
  }

  @Override
  public void stopClusteredSingleton() {
  }

  @Override
  public void produce() {
    Long cacheEntry = System.currentTimeMillis();
    logger.info("Produce new cache entry {} on node {}", cacheEntry,
        System.getProperty("jboss.node.name"));
    myAppCache.put(cacheEntry, cacheEntry);
  }

  @Schedule(hour = "*", minute = "*", second = "*")
  public void consume() {
    logger.info("Cache size on node {} = {}",
        System.getProperty("jboss.node.name"), myAppCache.size());
  }
}
