package org.hypik.test.jboss.eap7.server.ha;

import org.jboss.logging.Logger;
import org.jboss.msc.service.Service;
import org.jboss.msc.service.StartContext;
import org.jboss.msc.service.StartException;
import org.jboss.msc.service.StopContext;

import java.io.Serializable;
import java.util.concurrent.atomic.AtomicBoolean;

import javax.enterprise.inject.spi.CDI;

public class AbstractHAService<S extends IHAService> implements Service<String>, Serializable {
  /**
   * serial uid.
   */
  private static final long serialVersionUID = 7695519570740352042L;

  private static final Logger LOGGER = Logger.getLogger(AbstractHAService.class);

  private final Class<S>      serviceClazz;
  
  private S service;

  public AbstractHAService(Class<S> serviceClazz) {
    this.serviceClazz = serviceClazz;
  }

  /**
   * A flag whether the service is started.
   */
  private final AtomicBoolean started = new AtomicBoolean(false);

  @Override
  public String getValue() throws IllegalStateException, IllegalArgumentException {
    return System.getProperty("jboss.node.name");
  }

  @Override
  public void start(StartContext context) throws StartException {
    boolean cdiNotInitialized = true;
    while (cdiNotInitialized) {
      try {
        // What the hell with that ???
        cdiNotInitialized = CDI.current().getBeanManager() == null;
      } catch (Exception e) {
        LOGGER.debug("CDI not initialized " + e, e);
      }
    }
    if (!started.compareAndSet(false, true)) {
      throw new StartException("The service is still started!");
    }
    LOGGER.info("Starting HA singleton service "
        + serviceClazz.getSimpleName() + " on "
        + System.getProperty("jboss.node.name"));

    getLocalService().startClusteredSingleton();
  }

  @Override
  public void stop(StopContext context) {
    if (!started.compareAndSet(true, false)) {
      LOGGER.warn("The service " + this.getClass().getName() + " is not active!");
    } else {
      LOGGER.info("Stop  HA singleton service " + this.getClass().getName());
      getLocalService().stopClusteredSingleton();
    }
  }

  public synchronized S getLocalService() {
    if (service == null) {
      service = CDI.current().select(serviceClazz).get();
    }
    return service;
  }
}