package org.hypik.test.jboss.eap7.server.ha;

import org.hypik.test.jboss.eap7.server.ha.services.HAProducerTimer;
import org.hypik.test.jboss.eap7.server.ha.services.HATestService;
import org.jboss.msc.service.Service;
import org.jboss.msc.service.ServiceActivator;
import org.jboss.msc.service.ServiceActivatorContext;
import org.jboss.msc.service.ServiceBuilder.DependencyType;
import org.jboss.msc.service.ServiceController;
import org.jboss.msc.service.ServiceName;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.wildfly.clustering.singleton.SingletonServiceBuilderFactory;
import org.wildfly.clustering.singleton.SingletonServiceName;
import org.wildfly.clustering.singleton.election.SimpleSingletonElectionPolicy;

import java.io.Serializable;

public class HAServicesActivator implements ServiceActivator {
  private static final String CONTAINER_NAME = "server";

  private Object[][]          haServices     = {
      {HATestService.class,
        HATestService.SINGLETON_SERVICE_NAME,
        new ServiceName[] {
             ServiceName.of("jboss",
               "naming",
               "context",
               "java",
               "jboss",
               "infinispan",
               "container",
               "my-cache-container")
        }},
      {HAProducerTimer.class,
          HAProducerTimer.SINGLETON_SERVICE_NAME,
      new ServiceName[] {HATestService.SINGLETON_SERVICE_NAME}},
  };
  private static final Logger LOGGER         = LoggerFactory.getLogger(HAServicesActivator.class);

  @Override
  public void activate(ServiceActivatorContext context) {
    ServiceController<?> factoryService = context.getServiceRegistry().getRequiredService(
        SingletonServiceName
        .BUILDER.getServiceName(CONTAINER_NAME));

    SingletonServiceBuilderFactory factory = (SingletonServiceBuilderFactory)factoryService
        .getValue();

    for (Object[] haService : haServices) {
      Class<Service<Serializable>> serviceClass = (Class<Service<Serializable>>)haService[0];
      ServiceName serviceName = (ServiceName)haService[1];
      ServiceName[] dependencies = (ServiceName[])haService[2];
      LOGGER.info("Service " + serviceClass.getName() + "=" + serviceName + " will be installed!");

      Service<Serializable> service;
      try {
        service = serviceClass.newInstance();

        factory.createSingletonServiceBuilder(serviceName, service)
            .electionPolicy(new SimpleSingletonElectionPolicy())
            .build(context.getServiceTarget())
            .addDependencies(DependencyType.REQUIRED, dependencies)
            .setInitialMode(ServiceController.Mode.ACTIVE)
            .install();
      } catch (Exception e) {
        throw new RuntimeException(e);
      }
    }
  }
}