package org.hypik.test.jboss.eap7.server.timer;

import org.hypik.test.jboss.eap7.server.ha.IHAService;
import org.slf4j.Logger;

import javax.annotation.Resource;
import javax.ejb.Timer;
import javax.ejb.TimerService;
import javax.inject.Inject;

public class AbstractTimer implements IHAService {

  /**
   * serial uid.
   */
  private static final long serialVersionUID = -3234844228082125564L;

  @Inject
  private Logger                    logger;

  @Resource
  private TimerService timerService;

  private Timer timer;
  
  private final int interval;

  public AbstractTimer(int interval) {
    super();
    this.interval = interval;
  }

  @Override
  public void startClusteredSingleton() {
    logger.trace("Creating timer for {}", getClass());
    createTimer();
  }

  /**
   * Create the timer.
   */
  protected void createTimer() {
    this.timer = timerService.createTimer(interval, 
        "Timer for " + getClass());
  }

  @Override
  public void stopClusteredSingleton() {
    logger.trace("Stopping timer for {}", getClass());
    if (timer != null && timer.getTimeRemaining() > 0) {
      try {
        timer.cancel();
      } catch (Exception e) {
        logger.trace("Error while stopping timer " + e, e);
      }
    }
  }
}
